package de.intevation.mxd.reader;

import java.lang.Exception;

import org.w3c.dom.Element;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.IMarkerSymbol;
import com.esri.arcgis.display.SimpleMarkerSymbol;
import com.esri.arcgis.display.ArrowMarkerSymbol;
import com.esri.arcgis.display.CharacterMarkerSymbol;
import com.esri.arcgis.display.PictureMarkerSymbol;
import com.esri.arcgis.display.MultiLayerMarkerSymbol;

/**
 * Wrapper for marker symbol reader.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class MarkerSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(MarkerSymbolReader.class);

    private ISymbol symbol;
    private IMarkerSymbol markerSymbol;

    public MarkerSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor(ISymbol)");
        this.symbol = symbol;
        this.markerSymbol = null;
    }

    public MarkerSymbolReader(IMarkerSymbol symbol)
    throws Exception {
        logger.debug("contructor(ISymbol)");
        this.markerSymbol = symbol;
        this.symbol = null;
    }

    public MarkerSymbolReader() {
        logger.debug("contructor()");
        this.symbol = null;
        this.markerSymbol = null;
    }

    public Element read() {
        logger.debug("read()");
        ISymbolReader sreader = null;

        if(symbol != null) {
            try {
                if(symbol instanceof SimpleMarkerSymbol) {
                    sreader = new SimpleMarkerSymbolReader(symbol);
                }
                else if(symbol instanceof ArrowMarkerSymbol) {
                    sreader = new ArrowMarkerSymbolReader(symbol);
                }
                else if(symbol instanceof CharacterMarkerSymbol) {
                    sreader = new CharacterMarkerSymbolReader(symbol);
                }
                else if(symbol instanceof PictureMarkerSymbol) {
                    sreader = new PictureMarkerSymbolReader(symbol);
                }
                else if(symbol instanceof MultiLayerMarkerSymbol) {
                    sreader = new MultiLayerMarkerSymbolReader(symbol);
                }
                else {
                    logger.debug("The reader for type " + symbol.getClass().toString() +
                                 " is not implemented!");
                    return parent;
                }
            }
            catch(Exception e) {
                logger.error(
                    "Could not read the symbol " +
                    symbol.getClass().toString());
                return parent;

            }
        }
        else if(markerSymbol != null) {
            try {
                if(markerSymbol instanceof SimpleMarkerSymbol) {
                    sreader = new SimpleMarkerSymbolReader(markerSymbol);
                }
                else if(markerSymbol instanceof ArrowMarkerSymbol) {
                    sreader = new ArrowMarkerSymbolReader(markerSymbol);
                }
                else if(markerSymbol instanceof CharacterMarkerSymbol) {
                    sreader = new CharacterMarkerSymbolReader(markerSymbol);
                }
                else if(markerSymbol instanceof PictureMarkerSymbol) {
                    sreader = new PictureMarkerSymbolReader(markerSymbol);
                }
                else if(markerSymbol instanceof MultiLayerMarkerSymbol) {
                    sreader = new MultiLayerMarkerSymbolReader(markerSymbol);
                }
                else {
                    logger.debug("The reader for type " + symbol.getClass().toString() +
                                 " is not implemented!");
                    return parent;
                }
            }
            catch(Exception e) {
                logger.error(
                    "Could not read the symbol " +
                    symbol.getClass().toString());
                return parent;

            }
        }
        if (sreader != null) {
            sreader.setParent(parent);
            sreader.setUtil(util);
            sreader.read();
        }
        return parent;
    }

    public void setSymbol(ISymbol sym) {
        this.symbol = sym;
        this.markerSymbol = null;
    }

    public void setSymbol(IMarkerSymbol sym) {
        this.markerSymbol = sym;
        this.symbol = null;
    }

    public boolean canRead(ISymbol sym) {
        if(sym instanceof SimpleMarkerSymbol ||
           sym instanceof ArrowMarkerSymbol ||
           sym instanceof PictureMarkerSymbol ||
           sym instanceof CharacterMarkerSymbol ||
           sym instanceof MultiLayerMarkerSymbol) {
            return true;
        }
        else {
            return false;
        }
    }

    public boolean canRead(IMarkerSymbol sym) {
        if(sym instanceof SimpleMarkerSymbol ||
           sym instanceof ArrowMarkerSymbol ||
           sym instanceof PictureMarkerSymbol ||
           sym instanceof CharacterMarkerSymbol ||
           sym instanceof MultiLayerMarkerSymbol) {
            return true;
        }
        else {
            return false;
        }
    }

}
